﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Core;
using HeuristicLab.Common;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Optimization;
using HeuristicLab.Data;

namespace HeuristicLab.Problems.MetaOptimization {
  // todo: storable, name, descr, ...
  [StorableClass]
  public class ParameterConfigurationTree : ValueConfiguration {
    //[Storable]
    //protected EngineAlgorithm algorithm;
    //public EngineAlgorithm Algorithm {
    //  get { return algorithm; }
    //  set {
    //    if (algorithm != value) {
    //      if (algorithm != null) DeregisterAlgorithmEvents();
    //      algorithm = value;
    //      if (algorithm != null) RegisterAlgorithmEvents();
    //      OnAlgorithmChanged();
    //    }
    //  }
    //}

    [Storable]
    private DoubleValue quality;
    public DoubleValue Quality {
      get { return quality; }
      set {
        if (quality != value) {
          if (quality != null) DeregisterQualityEvents();
          quality = value;
          if (quality != null) RegisterQualityEvents();
          OnQualityChanged();
        }
      }
    }

    #region constructors and cloning
    public ParameterConfigurationTree(EngineAlgorithm algorithm) : base(algorithm, algorithm.GetType()) {
      this.Optimize = true; // root must always be optimized
      this.Quality = new DoubleValue();
      //this.algorithm = algorithm;
    }
    public ParameterConfigurationTree() {
      Initialize();
    }
    [StorableConstructor]
    protected ParameterConfigurationTree(bool deserializing) : base(deserializing) { }
    protected ParameterConfigurationTree(ParameterConfigurationTree original, Cloner cloner) : base(original, cloner) {
      this.quality = cloner.Clone(original.Quality);
      Initialize();
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new ParameterConfigurationTree(this, cloner);
    }
    [StorableHook(HookType.AfterDeserialization)]
    private void AfterDeserialization() {
      Initialize();
    }
    #endregion

    private void Initialize() {
      //if (algorithm != null) RegisterAlgorithmEvents();
      if (quality != null) RegisterQualityEvents();
    }

    #region Events
    //public event EventHandler AlgorithmChanged;
    //private void OnAlgorithmChanged() {
    //  var handler = AlgorithmChanged;
    //  if (handler != null) handler(this, EventArgs.Empty);
    //}
    public event EventHandler QualityChanged;
    private void OnQualityChanged() {
      var handler = QualityChanged;
      if (handler != null) handler(this, EventArgs.Empty);
    }

    //private void RegisterAlgorithmEvents() {
      
    //}
    //private void DeregisterAlgorithmEvents() {

    //}
    private void RegisterQualityEvents() {
      Quality.ValueChanged += new EventHandler(Quality_ValueChanged);
    }
    private void DeregisterQualityEvents() {
      Quality.ValueChanged -= new EventHandler(Quality_ValueChanged);
    }

    private void Quality_ValueChanged(object sender, EventArgs e) {
      OnQualityChanged();
    }
    #endregion
  }
}
