﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Data;
using HeuristicLab.Common;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Core;

namespace HeuristicLab.Problems.MetaOptimization {
  [StorableClass]
  public class IntValueRange : Range<IntValue> {

    public IntValueRange(IntValue lowerBound, IntValue upperBound, IntValue stepSize) : base(lowerBound, upperBound, stepSize) { }
    public IntValueRange() { }
    [StorableConstructor]
    protected IntValueRange(bool deserializing) : base(deserializing) { }
    protected IntValueRange(IntValueRange original, Cloner cloner) : base(original, cloner) { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new IntValueRange(this, cloner);
    }

    public override IntValue GetRandomValue(IRandom random) {
      int val;
      do {
        val = random.Next(LowerBound.Value / StepSize.Value, UpperBound.Value / StepSize.Value + 1) * StepSize.Value;
      } while (val < LowerBound.Value || val > UpperBound.Value);
      return new IntValue(val);
    }
  }
}
